// Generated by CoffeeScript 1.10.0
(function() {
  var CN, PASS_PATH, PRI_PATH, PUB_PATH, SSH_PATH, crypto, fs, os, path, pkg, pr, replaceHome, rl;

  pkg = require('./package.json');

  crypto = require('crypto');

  fs = require('fs');

  path = require('path');

  os = require('os');

  CN = require('constants');

  pr = require('commander');

  rl = require('readline');

  SSH_PATH = path.resolve(os.homedir(), '.ssh');

  PRI_PATH = path.resolve(SSH_PATH, 'rsa.pem');

  PUB_PATH = path.resolve(SSH_PATH, 'rsa_pub.pem');

  PASS_PATH = path.resolve(SSH_PATH, 'passkey');

  replaceHome = function(path) {
    return path.replace(/^\~/, os.homedir());
  };

  pr.command('encpriv <text>').description('Encrypt with private key').option('-e, --encoding [name]', 'Set encoding for exiting cipher', 'hex').option('-R, --private [path]', 'Set path to private key', PRI_PATH).action(function(text, options) {
    var buf, enc, key;
    key = fs.readFileSync(replaceHome(options["private"]), 'utf8');
    buf = new Buffer(text, 'utf8');
    enc = crypto.privateEncrypt(key, buf);
    console.log('\nYour cipher:');
    return console.log(enc.toString(options.encoding));
  });

  pr.command('encpub <text>').description('Encrypt with public key').option('-e, --encoding [name]', 'Set encoding for exiting cipher', 'hex').option('-P, --public [path]', 'Set path to public key', PUB_PATH).action(function(text, options) {
    var buf, enc, key;
    key = fs.readFileSync(replaceHome(options["public"]), 'utf8');
    buf = new Buffer(text, 'utf8');
    enc = crypto.publicEncrypt(key, buf);
    console.log('\nYour cipher:');
    return console.log(enc.toString(options.encoding));
  });

  pr.command('decpriv <encrypted>').description('Decrypt with private key by encrypted string').option('-e, --encoding [name]', 'Set encoding for encrypted string', 'hex').option('-R, --private [path]', 'Set path to private key', PRI_PATH).action(function(encrypted, options) {
    var buf, dec, key;
    key = fs.readFileSync(replaceHome(options["private"]), 'utf8');
    buf = new Buffer(encrypted, options.encoding);
    dec = crypto.privateDecrypt(key, buf);
    console.log('\nDecrypted:');
    return console.log(dec.toString('utf8'));
  });

  pr.command('decpub <encrypted>').description('Decrypt with public key by encrypted string').option('-e, --encoding [name]', 'Set encoding for encrypted string', 'hex').option('-P, --public [path]', 'Set path to public key', PUB_PATH).action(function(encrypted, options) {
    var buf, dec, key;
    key = fs.readFileSync(replaceHome(options["public"]), 'utf8');
    buf = new Buffer(encrypted, options.encoding);
    dec = crypto.publicDecrypt(key, buf);
    console.log('\nDecrypted:');
    return console.log(dec.toString('utf8'));
  });

  pr.command('enc').description('Encrypt with cipher by text or file').option('-f, --file [path]', 'Send file for encryption').option('-t, --text [text]', 'Send text for encryption').option('-e, --encoding [name]', 'Set encoding for exiting cipher', 'hex').option('-K, --passkey [path]', 'Set path to passkey for encryption', PASS_PATH).action(function(options) {
    var cipher, enc, passkey, stream;
    if (!options.text && !options.file) {
      return console.error('No text or file');
    }
    passkey = fs.readFileSync(replaceHome(options.passkey));
    cipher = crypto.createCipher('aes256', passkey);
    if (options.text) {
      enc = cipher.update(options.text, 'utf8', options.encoding);
      enc += cipher.final(options.encoding);
      console.log('\nYour cipher:');
      return console.log(enc);
    } else {
      enc = '';
      stream = fs.createReadStream(replaceHome(options.file));
      cipher.on('readable', function() {
        var data;
        data = cipher.read();
        if (data) {
          return enc += data.toString(options.encoding);
        }
      });
      cipher.on('end', function() {
        console.log('\nYour cipher:');
        return console.log(enc);
      });
      return stream.pipe(cipher);
    }
  });

  pr.command('dec <encrypted>').description('Decrypt with cipher by encrypted string').option('-e, --encoding [name]', 'Set encoding for encrypted string', 'hex').option('-K, --passkey [path]', 'Set path to passkey for decryption', PASS_PATH).action(function(encrypted, options) {
    var cipher, dec, passkey;
    passkey = fs.readFileSync(replaceHome(options.passkey));
    cipher = crypto.createDecipher('aes256', passkey);
    dec = cipher.update(encrypted, options.encoding, 'utf8');
    dec += cipher.final('utf8');
    console.log('\nDecrypted:');
    return console.log(dec);
  });

  pr.command('sign <file_path>').description('Create sign for file').option('-S, --save [path]', 'Save signature to path').option('-e, --encoding [name]', 'Set encoding for returning passkey (if no save flag)', 'hex').option('-R, --private [path]', 'Set path to public key', PRI_PATH).action(function(file, options) {
    var key, sign, signature;
    key = fs.readFileSync(replaceHome(options["private"]));
    sign = crypto.createSign('RSA-SHA256');
    file = fs.readFileSync(replaceHome(file));
    sign.update(file);
    signature = sign.sign(key);
    if (options.save) {
      fs.writeFileSync(replaceHome(options.save), signature);
      return console.log("Saved on", replaceHome(options.save));
    } else {
      console.log("Your signature:\n");
      return console.log(signature.toString(options.encoding));
    }
  });

  pr.command('verify <file_path>').description('Verify signed file').option('-f, --file [path]', 'Send path to signature file').option('-t, --text [text]', 'Send text of signature').option('-e, --encoding [name]', 'Set encoding signature text (if text sended', 'hex').option('-P, --public [path]', 'Set path to public key', PUB_PATH).action(function(file, options) {
    var key, signature, verified, verify;
    key = fs.readFileSync(replaceHome(options["public"]));
    verify = crypto.createVerify('RSA-SHA256');
    file = fs.readFileSync(replaceHome(file));
    if (options.text) {
      signature = new Buffer(options.text, options.encoding);
    } else if (options.file) {
      signature = fs.readFileSync(replaceHome(options.file));
    }
    verify.update(file);
    verified = verify.verify(key, signature);
    if (verified) {
      return console.log('Verified');
    } else {
      return console.log('Unverified');
    }
  });

  pr.command('dhmake').description('Make DH passkey').option('-s, --slave', 'Command for second client', false).option('-S, --save [path]', 'Save passkey path (show in console if not sended)').option('-e, --encoding [name]', 'Set encoding for returning passkey (if no save flag)', 'hex').action(function(options) {
    var ask;
    ask = rl.createInterface({
      input: process.stdin,
      output: process.stdout
    });
    if (!options.slave) {
      return ask.question('Enter prime length: ', function(length) {
        var dh;
        dh = crypto.createDiffieHellman(Number(length));
        dh.generateKeys();
        console.log("Your master prime:", dh.getPrime('hex'));
        console.log("Your master generator:", dh.getGenerator('hex'));
        console.log("Your master public key:", dh.getPublicKey('hex'));
        return ask.question('Enter slave public key: ', function(key) {
          var passkey;
          ask.close();
          passkey = dh.computeSecret(key, 'hex');
          if (options.save) {
            fs.writeFileSync(replaceHome(options.save), passkey);
            return console.log("Saved on", replaceHome(options.save));
          } else {
            console.log("\nYour passkey in (" + options.encoding + "):");
            return console.log(passkey.toString(options.encoding));
          }
        });
      });
    } else {
      return ask.question('Enter master prime (in hex): ', function(prime) {
        return ask.question('Enter master generator (in hex): ', function(generator) {
          var dh;
          dh = crypto.createDiffieHellman(prime, 'hex', generator, 'hex');
          dh.generateKeys();
          console.log("Your slave public key", dh.getPublicKey('hex'));
          return ask.question('Enter master public key: ', function(key) {
            var passkey;
            ask.close();
            passkey = dh.computeSecret(key, 'hex');
            if (options.save) {
              fs.writeFileSync(replaceHome(options.save), passkey);
              return console.log("Saved on", replaceHome(options.save));
            } else {
              console.log("\nYour passkey in (" + options.encoding + "):");
              return console.log(passkey.toString(options.encoding));
            }
          });
        });
      });
    }
  });

  pr.command('ecdhmake').option('-c, --curve [curve]', 'Set Elliptic Curve name', 'secp521r1').option('-S, --save [path]', 'Save passkey path (show in console if not sended)').option('-e, --encoding [name]', 'Set encoding for returning passkey (if no save flag)', 'hex').description('Make ECDH passkey').action(function(options) {
    var ask, dh;
    ask = rl.createInterface({
      input: process.stdin,
      output: process.stdout
    });
    dh = crypto.createECDH(options.curve);
    dh.generateKeys();
    console.log("Your public key:", dh.getPublicKey('hex'));
    return ask.question('Enter other public key: ', function(key) {
      var passkey;
      ask.close();
      passkey = dh.computeSecret(key, 'hex');
      if (options.save) {
        fs.writeFileSync(replaceHome(options.save), passkey);
        return console.log("Saved on", replaceHome(options.save));
      } else {
        console.log("\nYour passkey in (" + options.encoding + "):");
        return console.log(passkey.toString(options.encoding));
      }
    });
  });

  pr.version(pkg.version).parse(process.argv);

}).call(this);
